"""Main of Epson projector module."""
import logging
import time

import aiohttp
import async_timeout

from .const import (ACCEPT_ENCODING, ACCEPT_HEADER, ALL,
                    EPSON_KEY_COMMANDS, HTTP_OK, INV_SOURCES, SOURCE,
                    TIMEOUT_TIMES, TURN_OFF, TURN_ON)

_LOGGER = logging.getLogger(__name__)


class Projector:
    """
    Epson projector class.

    Control your projector with Python.
    """

    def __init__(self, host, websession, port=80, encryption=False):
        """
        Epson Projector controller.

        :param str ip:          IP address of Projector
        :param int port:        Port to connect to. Default 80.
        :param bool encryption: User encryption to connect

        """
        self._host = host
        self._port = port
        self._encryption = encryption
        self._powering_on = False
        http_proto = 'https' if self._encryption else 'http'
        self._http_url = '{http_proto}://{host}:{port}/cgi-bin/'.format(
            http_proto=http_proto,
            host=self._host,
            port=self._port)
        referer = "{http_proto}://{host}:{port}/cgi-bin/webconf".format(
            http_proto=http_proto,
            host=self._host,
            port=self._port)
        self._headers = {
            "Accept-Encoding": ACCEPT_ENCODING,
            "Accept": ACCEPT_HEADER,
            "Referer": referer
        }
        self.websession = websession
        self.__initLock()

    def __initLock(self):
        """Init lock for sending request to projector when it is busy."""
        self._isLocked = False
        self._timer = 0
        self._operation = False

    def __setLock(self, command):
        """Set lock on requests."""
        if command in (TURN_ON, TURN_OFF):
            self._operation = command
        elif command in INV_SOURCES:
            self._operation = SOURCE
        else:
            self._operation = ALL
        self._isLocked = True
        self._timer = time.time()

    def __unLock(self):
        """Unlock sending requests to projector."""
        self._operation = False
        self._timer = 0
        self._isLocked = False

    def __checkLock(self):
        """
        Lock checking.

        Check if there is lock pending and check if enough time
        passed so requests can be unlocked.
        """
        if self._isLocked:
            if (time.time() - self._timer) > TIMEOUT_TIMES[self._operation]:
                self.__unLock()
                return False
            return True
        return False

    async def get_property(self, command):
        """Get property state from device."""
        _LOGGER.debug("Getting property %s", command)
        if self.__checkLock():
            return False
        timeout = self.__get_timeout(command)
        response = await self.send_request(
            timeout=timeout,
            params=EPSON_KEY_COMMANDS[command],
            type='json_query')
        resp = await response.json()
        reply_code = resp['projector']['feature']['reply']
        return reply_code

    async def send_command(self, command):
        """Send command to Epson."""
        _LOGGER.debug("Sending command to projector %s", command)
        if self.__checkLock():
            return False
        self.__setLock(command)
        response = await self.send_request(
            timeout=self.__get_timeout(command),
            params=EPSON_KEY_COMMANDS[command],
            type='directsend',
            command=command)
        return response

    async def send_request(self, params, timeout,
                           type='json_query', command=False):
        """Send request to Epson."""
        try:
            with async_timeout.timeout(timeout):
                url = '{url}{type}'.format(
                    url=self._http_url,
                    type=type)
                async with self.websession.get(
                    url=url, params=params,
                        headers=self._headers) as response:
                    if response.status != HTTP_OK:
                        _LOGGER.warning(
                            "Error message %d from Epson.", response.status)
                        return False
                    if command == TURN_ON and self._powering_on:
                        self._powering_on = False
                    return response
        except aiohttp.ClientError:
            _LOGGER.error("Error request")
            return False

    def __get_timeout(self, command):
        if command in TIMEOUT_TIMES:
            return TIMEOUT_TIMES[command]
        else:
            return TIMEOUT_TIMES['ALL']



"""Setup of Epson projector module."""

import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

with open('epson_projector/version.py') as fh:
    exec(fh.read())

setuptools.setup(
    name="epson_projector",
    version=__version__,
    author="Paweł Szafer",
    author_email="pszafer@gmail.com",
    description="Epson projector support for Python",
    license="MIT",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/pszafer/epson_projector",
    install_requires=list(val.strip() for val in open('requirements.txt')),
    packages=['epson_projector'],
    keywords=['epson', 'projector'],
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ),
)



"""Const helpers of Epson projector module."""

HTTP_OK = 200

ACCEPT_ENCODING = "gzip, deflate"
ACCEPT_HEADER = "application/json, text/javascript"

POWER = "PWR"
CMODE = "CMODE"
SOURCE = "SOURCE"
VOLUME = "VOLUME"
MUTE = "MUTE"
VOL_UP = "VOL_UP"
VOL_DOWN = "VOL_DOWN"
PLAY = "PLAY"
PAUSE = "PAUSE"
FAST = "FAST"
BACK = "BACK"
TURN_ON = "TURN_ON"
TURN_OFF = "TURN_OFF"
ALL = "ALL"

EPSON_CODES = {
    'PWR': '01'
}

EPSON_KEY_COMMANDS = {
    "TURN_ON": [('KEY', '3B')],
    "TURN_OFF": [('KEY', '3B'), ('KEY', '3B')],
    "HDMILINK": [('jsoncallback', 'HDMILINK?')],
    "PWR": [('jsoncallback', 'PWR?')],
    "SOURCE": [('jsoncallback', 'SOURCE?')],
    "CMODE": [('jsoncallback', 'CMODE?')],
    "VOLUME": [('jsoncallback', 'VOL?')],
    "CMODE_AUTO": [('CMODE', '00')],
    "CMODE_CINEMA": [('CMODE', '15')],
    "CMODE_NATURAL": [('CMODE', '07')],
    "CMODE_BRIGHT": [('CMODE', '0C')],
    "CMODE_DYNAMIC": [('CMODE', '06')],
    "CMODE_3DDYNAMIC": [('CMODE', '18')],
    "CMODE_3DCINEMA": [('CMODE', '17')],
    "CMODE_3DTHX": [('CMODE', '19')],
    "CMODE_BWCINEMA": [('CMODE', '20')],
    "CMODE_ARGB": [('CMODE', '21')],
    "CMODE_DCINEMA": [('CMODE', '22')],
    "CMODE_THX": [('CMODE', '13')],
    "CMODE_GAME": [('CMODE', '0D')],
    "CMODE_STAGE": [('CMODE', '16')],
    "CMODE_AUTOCOLOR": [('CMODE', 'C1')],
    "CMODE_XV": [('CMODE', '0B')],
    "CMODE_THEATRE": [('CMODE', '05')],
    "CMODE_THEATREBLACK": [('CMODE', '09')],
    "CMODE_THEATREBLACK2": [('CMODE', '0A')],
    "VOL_UP": [('KEY', '56')],
    "VOL_DOWN": [('KEY', '57')],
    "MUTE": [('KEY', 'D8')],
    "HDMI1": [('KEY', '4D')],
    "HDMI2": [('KEY', '40')],
    "PC": [('KEY', '44')],
    "VIDEO": [('KEY', '46')],
    "USB": [('KEY', '85')],
    "LAN": [('KEY', '53')],
    "WFD": [('KEY', '56')],
    "PLAY": [('KEY', 'D1')],
    "PAUSE": [('KEY', 'D3')],
    "STOP": [('KEY', 'D2')],
    "BACK": [('KEY', 'D4')],
    "FAST": [('KEY', 'D5')],
}

TIMEOUT_TIMES = {
    'TURN_ON': 40,
    'TURN_OFF': 10,
    'SOURCE': 5,
    'ALL': 3
}

DEFAULT_SOURCES = {
    'HDMI1': 'HDMI1',
    'HDMI2': 'HDMI2',
    'PC': 'PC',
    'VIDEO': 'VIDEO',
    'USB': 'USB',
    'LAN': 'LAN',
    'WFD': 'WiFi Direct'
}

SOURCE_LIST = {
    '30': 'HDMI1',
    '10': 'PC',
    '40': 'VIDEO',
    '52': 'USB',
    '53': 'LAN',
    '56': 'WDF',
    'A0': 'HDMI2'
}

INV_SOURCES = {v: k for k, v in DEFAULT_SOURCES.items()}

CMODE_LIST = {
    '00': 'Auto',
    '15': 'Cinema',
    '07': 'Natural',
    '0C': 'Bright Cinema/Living',
    '06': 'Dynamic',
    '17': '3D Cinema',
    '18': '3D Dynamic',
    '19': '3D THX',
    '20': 'B&W Cinema',
    '21': 'Adobe RGB',
    '22': 'Digital Cinema',
    '13': 'THX',
    '0D': 'Game',
    '16': 'Stage',
    'C1': 'AutoColor',
    '0B': 'x.v. color',
    '05': 'Theatre',
    '09': 'Theatre Black 1/HD',
    '0A': 'Theatre Black 2/Silver Screen'
}

CMODE_LIST_SET = {
    'cinema': 'CMODE_CINEMA',
    'natural': 'CMODE_NATURAL',
    'bright cinema': 'CMODE_BRIGHT',
    'dynamic': 'CMODE_DYNAMIC',
    '3ddynamic': 'CMODE_3DDYNAMIC',
    '3dcinema': 'CMODE_3DCINEMA',
    'auto': 'CMODE_AUTO',
    '3dthx': 'CMODE_3DTHX',
    'bwcinema': 'CMODE_BWCINEMA',
    'adobe rgb': 'CMODE_ARGB',
    'digital cinema': 'CMODE_DCINEMA',
    'thx': 'CMODE_THX',
    'game': 'CMODE_GAME',
    'stage': 'CMODE_STAGE',
    'autocolor': 'CMODE_AUTOCOLOR',
    'xv': 'CMODE_XV',
    'theatre': 'CMODE_THEATRE',
    'theatre black': 'CMODE_THEATREBLACK',
    'theatre black 2': 'CMODE_THEATREBLACK2'
}
